package Lingua::Ab::H;

use strict;
use warnings;

use base qw( Lingua::Abbreviate::Hierarchy );

=head1 NAME

Lingua::Ab::H - Shorter alias for Lingua::Abbreviate::Hierarchy

=head1 VERSION

This document describes Lingua::Ab::H version 0.04

=cut

our $VERSION = '0.04';

=head1 INTERFACE

See L<Lingua::Abbreviate::Hierarchy>.

=cut

"Ceci n'est pas 'Modern Perl'";

# vim:ts=2:sw=2:sts=2:et:ft=perl
